import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:widgetbook/src/knobs/knobs.dart';
import 'package:widgetbook/src/knobs/nullable_checkbox.dart';
import 'package:widgetbook_core/widgetbook_core.dart' as core;

class BoolKnob extends Knob<bool> {
  BoolKnob({
    required super.label,
    super.description,
    required super.value,
  });

  @override
  Widget build(BuildContext context) => core.BoolKnob(
        name: label,
        description: description,
        value: value,
        onChanged: (value) {
          context.read<KnobsNotifier>().update(label, value);
        },
      );
}

class NullableBoolKnob extends Knob<bool?> {
  NullableBoolKnob({
    required super.label,
    super.description,
    required super.value,
  });

  @override
  Widget build(BuildContext context) => core.NullableBoolKnob(
        name: label,
        description: description,
        value: value,
        onChanged: (value) {
          context.read<KnobsNotifier>().update(label, value);
        },
      );
}

class BooleanKnobWidget extends StatefulWidget {
  const BooleanKnobWidget({
    super.key,
    required this.label,
    required this.description,
    required this.value,
    this.nullable = false,
  });

  final String label;
  final String? description;
  final bool? value;
  final bool nullable;

  @override
  State<BooleanKnobWidget> createState() => _BooleanKnobWidgetState();
}

class _BooleanKnobWidgetState extends State<BooleanKnobWidget> {
  bool value = false;

  @override
  void initState() {
    super.initState();
    if (widget.value != null) {
      value = widget.value!;
    }
  }

  @override
  Widget build(BuildContext context) {
    final disabled = widget.value == null;
    return KnobWrapper(
      title: widget.label,
      description: widget.description,
      nullableCheckbox: widget.nullable
          ? NullableCheckbox<bool?>(
              cachedValue: value,
              value: widget.value,
              label: widget.label,
            )
          : null,
      child: SizedBox(
        child: CupertinoSwitch(
          key: Key('${widget.label}-switchTileKnob'),
          value: value,
          activeColor: Theme.of(context).colorScheme.primary,
          onChanged: disabled
              ? null
              : (v) {
                  setState(() {
                    value = v;
                  });
                  context.read<KnobsNotifier>().update(widget.label, v);
                },
        ),
      ),
    );
  }
}
