import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:widgetbook/src/knobs/knobs.dart';
import 'package:widgetbook/src/knobs/nullable_checkbox.dart';
import 'package:widgetbook_core/widgetbook_core.dart' as core;

class SliderKnob extends Knob<double> {
  SliderKnob({
    required super.label,
    super.description,
    required super.value,
    this.max = 1,
    this.min = 0,
    this.divisions,
  });

  final double max;
  final double min;
  final int? divisions;

  @override
  Widget build(BuildContext context) => core.SliderKnob(
        name: label,
        description: description,
        min: min,
        max: max,
        divisions: divisions,
        value: value,
      );
}

class NullableSliderKnob extends Knob<double?> {
  NullableSliderKnob({
    required super.label,
    super.description,
    required double super.value,
    this.max = 1,
    this.min = 0,
    this.divisions,
  });

  final double max;
  final double min;
  final int? divisions;

  @override
  Widget build(BuildContext context) => core.NullableSliderKnob(
        name: label,
        description: description,
        min: min,
        max: max,
        divisions: divisions,
        value: value,
      );
}

class SliderKnobWidget extends StatefulWidget {
  const SliderKnobWidget({
    super.key,
    required this.label,
    required this.description,
    required this.value,
    required this.max,
    required this.min,
    required this.divisions,
    this.nullable = false,
  });

  final String label;
  final String? description;
  final double? value;

  final double max;
  final double min;
  final int? divisions;
  final bool nullable;

  @override
  State<SliderKnobWidget> createState() => _SliderKnobWidgetState();
}

class _SliderKnobWidgetState extends State<SliderKnobWidget> {
  double _value = 0;
  @override
  void initState() {
    super.initState();
    if (widget.value != null) {
      _value = widget.value!;
    }
  }

  @override
  Widget build(BuildContext context) {
    final disabled = widget.value == null;
    return KnobWrapper(
      title: '${widget.label} (${_value.toStringAsFixed(2)})',
      description: widget.description,
      nullableCheckbox: widget.nullable
          ? NullableCheckbox<double?>(
              cachedValue: _value,
              value: widget.value,
              label: widget.label,
            )
          : null,
      child: Slider(
        key: Key('${widget.label}-sliderKnob'),
        value: _value,
        min: widget.min,
        max: widget.max,
        divisions: widget.divisions,
        onChanged: disabled
            ? null
            : (v) {
                setState(() {
                  _value = v;
                });
                context.read<KnobsNotifier>().update(widget.label, v);
              },
        label: _value.toStringAsFixed(2),
      ),
    );
  }
}
